export function noop(defaultValue) {
    return {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        set: (value) => { },
        initialValue: defaultValue,
        defaultValue,
    };
}
function createStorage(storage, key, defaultValue, options = {}) {
    if (typeof window === 'undefined') {
        return noop(defaultValue);
    }
    const { serialize = JSON.stringify, deserialize = (value) => (value ? JSON.parse(value) : defaultValue), initialize = (value) => value, syncTabs = false, } = options;
    const internalStorage = window[storage];
    const get = () => {
        const value = internalStorage.getItem(key);
        return deserialize(value);
    };
    return {
        set: (value) => internalStorage.setItem(key, serialize(value)),
        initialValue: initialize(get()),
        defaultValue: defaultValue,
        event: syncTabs && storage === 'localStorage'
            ? {
                root: (listener) => {
                    ensureRootListener();
                    const onStorageChange = (value) => {
                        listener(deserialize(value));
                    };
                    addRootStorageListener(key, onStorageChange);
                    return () => removeRootStorageListener(key);
                },
                component: (listener) => {
                    const onStorageChange = (event) => {
                        if (event.key !== key)
                            return;
                        listener(deserialize(event.newValue));
                    };
                    window.addEventListener('storage', onStorageChange);
                    return () => window.removeEventListener('storage', onStorageChange);
                },
            }
            : undefined,
    };
}
export function createLocalStorage(key, defaultValue, options) {
    return createStorage('localStorage', key, defaultValue, options);
}
export function createSessionStorage(key, defaultValue, options) {
    return createStorage('sessionStorage', key, defaultValue, options);
}
export const rootStorageListeners = new Map();
export let isRootListenerEnabled = false;
export function onStorageChange(event) {
    if (!event.key)
        return;
    const listener = rootStorageListeners.get(event.key);
    if (!listener)
        return;
    listener(event.newValue);
}
function ensureRootListener() {
    if (!isRootListenerEnabled && typeof window !== 'undefined') {
        window.addEventListener('storage', onStorageChange);
        isRootListenerEnabled = true;
    }
}
function addRootStorageListener(key, listener) {
    rootStorageListeners.set(key, listener);
}
function removeRootStorageListener(key) {
    rootStorageListeners.delete(key);
}
